@@ -135,7 +135,7 @@
 #endif
 
 typedef enum {
-    OPTION_NOACCEL,
+    OPTION_ACCEL,
     OPTION_SW_CURSOR,
     OPTION_PAGE_FLIP,
     OPTION_EXA_PIXMAPS,
@@ -235,6 +235,12 @@ typedef enum {
 	(info->ChipFamily == CHIP_FAMILY_RS300) || \
 	(info->ChipFamily == CHIP_FAMILY_R200))
 
+#define CURSOR_WIDTH	64
+#define CURSOR_HEIGHT	64
+
+#define CURSOR_WIDTH_CIK	128
+#define CURSOR_HEIGHT_CIK	128
+
 struct radeon_exa_pixmap_priv {
     struct radeon_bo *bo;
     uint32_t tiling_flags;
@@ -425,6 +431,7 @@ typedef struct {
     Bool	      exa_pixmaps;
     Bool              exa_force_create;
     XF86ModReqInfo    exaReq;
+    Bool              is_fast_fb; /* use direct mapping for fast fb access */
 
     unsigned int xv_max_width;
     unsigned int xv_max_height;
@@ -453,6 +460,7 @@ typedef struct {
     uint64_t vram_size;
     uint64_t gart_size;
     drmmode_rec drmmode;
+    Bool drmmode_inited;
     /* r6xx+ tile config */
     Bool have_tiling_info;
     uint32_t tile_config;
@@ -471,6 +479,10 @@ typedef struct {
 
     /* Perform vsync'ed SwapBuffers? */
     Bool swapBuffersWait;
+
+    /* cursor size */
+    int cursor_w;
+    int cursor_h;
 } RADEONInfoRec, *RADEONInfoPtr;
 
 /* radeon_accel.c */
@@ -485,9 +497,6 @@ extern void RADEONWaitForVLine(ScrnInfoPtr pScrn, PixmapPtr pPix,
 			       xf86CrtcPtr crtc, int start, int stop);
 
 
-/* radeon_driver.c */
-extern RADEONEntPtr RADEONEntPriv(ScrnInfoPtr pScrn);
-
 /* radeon_exa.c */
 extern unsigned eg_tile_split(unsigned tile_split);
 extern Bool radeon_transform_is_affine_or_scaled(PictTransformPtr t);
@@ -515,7 +524,10 @@ extern void radeon_cs_flush_indirect(ScrnInfoPtr pScrn);
 extern void radeon_ddx_cs_start(ScrnInfoPtr pScrn,
 				int num, const char *file,
 				const char *func, int line);
-void radeon_kms_update_vram_limit(ScrnInfoPtr pScrn, int new_fb_size);
+void radeon_kms_update_vram_limit(ScrnInfoPtr pScrn, uint32_t new_fb_size);
+extern RADEONEntPtr RADEONEntPriv(ScrnInfoPtr pScrn);
+
+drmVBlankSeqType radeon_populate_vbl_request_type(xf86CrtcPtr crtc);
 
 #if XF86_CRTC_VERSION >= 5
 #define RADEON_PIXMAP_SHARING 1
@@ -562,16 +574,20 @@ static inline void radeon_set_pixmap_bo(PixmapPtr pPix, struct radeon_bo *bo)
 	    if (priv->bo)
 		radeon_bo_unref(priv->bo);
 
-	    free(priv);
-	    priv = NULL;
+	    if (!bo) {
+		free(priv);
+		priv = NULL;
+	    }
 	}
 
 	if (bo) {
 	    uint32_t pitch;
 
-	    priv = calloc(1, sizeof (struct radeon_pixmap));
-	    if (priv == NULL)
-		goto out;
+	    if (!priv) {
+		priv = calloc(1, sizeof (struct radeon_pixmap));
+		if (!priv)
+		    goto out;
+	    }
 
 	    radeon_bo_ref(bo);
 	    priv->bo = bo;
@@ -770,14 +786,21 @@ static __inline__ void RADEON_SYNC(RADEONInfoPtr info, ScrnInfoPtr pScrn)
 }
 
 enum {
-    RADEON_CREATE_PIXMAP_DRI2 = 0x08000000,
-    RADEON_CREATE_PIXMAP_TILING_MACRO = 0x10000000,
-    RADEON_CREATE_PIXMAP_TILING_MICRO = 0x20000000,
-    RADEON_CREATE_PIXMAP_DEPTH = 0x40000000, /* for r200 */
-    RADEON_CREATE_PIXMAP_SZBUFFER = 0x80000000, /* for eg */
-    RADEON_CREATE_PIXMAP_TILING_MICRO_SQUARE = 0x8000000,
+    RADEON_CREATE_PIXMAP_DRI2			= 0x04000000,
+    RADEON_CREATE_PIXMAP_TILING_MICRO_SQUARE	= 0x08000000,
+    RADEON_CREATE_PIXMAP_TILING_MACRO		= 0x10000000,
+    RADEON_CREATE_PIXMAP_TILING_MICRO		= 0x20000000,
+    RADEON_CREATE_PIXMAP_DEPTH			= 0x40000000, /* for r200 */
+    RADEON_CREATE_PIXMAP_SZBUFFER		= 0x80000000, /* for eg */
 };
 
+#define RADEON_CREATE_PIXMAP_TILING_FLAGS	\
+    (RADEON_CREATE_PIXMAP_TILING_MICRO_SQUARE |	\
+     RADEON_CREATE_PIXMAP_TILING_MACRO |	\
+     RADEON_CREATE_PIXMAP_TILING_MICRO |	\
+     RADEON_CREATE_PIXMAP_DEPTH |		\
+     RADEON_CREATE_PIXMAP_SZBUFFER)
+
 
 /* Compute log base 2 of val. */
 static __inline__ int
